@@ -27,7 +27,6 @@
 
 #include "dialog_glom.h"
 
-#include "navigation/box_databases.h"
 #include "navigation/box_tables.h"
 
 #include "mode_data/notebook_data.h"
@@ -35,16 +34,20 @@
 
 #include "mode_design/dialog_fields.h"
 #include "mode_design/dialog_relationships.h"
+#include "dialog_connection.h"
+
+#include "mode_data/box_data_list_related.h" //only for m_HackToFixLinkerError.
 
 class Frame_Glom :
   public PlaceHolder,
-  public Bakery::View_Composite<Document_Glom>
+  //public Bakery::View_Composite<Document_Glom>,
+  public Base_DB //Inherits from View_Composite.
 {
 public: 
   Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
   virtual ~Frame_Glom();
 
-  virtual void on_box_databases_selected(Glib::ustring strName);
+  virtual void set_databases_selected(const Glib::ustring& strName);
   virtual void on_box_tables_selected(Glib::ustring strName);
 
   virtual void on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction>& action);
@@ -53,8 +56,8 @@ public:
   virtual void on_menu_Mode_Data();
   virtual void on_menu_Mode_Find();
 
-  virtual void on_menu_Navigate_Database();
-  virtual void do_menu_Navigate_Database(bool bUseList = true);
+  //virtual void on_menu_Navigate_Database();
+  //virtual void do_menu_Navigate_Database(bool bUseList = true);
   virtual void on_menu_Navigate_Table();
   virtual void do_menu_Navigate_Table(bool open_default = false);
 
@@ -78,6 +81,14 @@ public:
   enumModes m_Mode;
   enumModes m_Mode_Previous; // see comments in set_mode_widget().
 
+  static void show_ok_dialog(const Glib::ustring& title, const Glib::ustring& message, Gtk::Window& parent);
+
+  //Show the dialog to request the password, and check whether it works.
+  virtual bool connection_request_password_and_attempt();
+  
+  ///Create the database for new documents, showing the Connection dialog
+  virtual bool create_database(const Glib::ustring& database_name, bool request_password = true);
+  
 protected:
 
   //virtual void set_document(Document_Glom* pDocument); //override
@@ -92,7 +103,6 @@ protected:
   virtual Gtk::Window* get_app_window();
   virtual const Gtk::Window* get_app_window() const;
   
-  virtual void show_ok_dialog(const Glib::ustring& title, const Glib::ustring& message);
   virtual void alert_no_table();
   
   //Signal handlers:
@@ -109,19 +119,23 @@ protected:
     
   PlaceHolder* m_pBox_Mode; //Contains e.g. design mode notebook.
   
-  Box_DataBases* m_pBox_Databases;
   Box_Tables* m_pBox_Tables;
 
   Notebook_Data m_Notebook_Data;
   Notebook_Find m_Notebook_Find;
 
   //Navigation:
-  Dialog_Glom* m_pDialog_Databases;
   Dialog_Glom* m_pDialog_Tables;
 
   //Developer:
   Dialog_Fields* m_pDialog_Fields;
   Dialog_Relationships* m_pDialog_Relationships;
+
+  Dialog_Connection* m_pDialogConnection;
+  Gtk::Dialog* m_pDialogConnectionFailed;
+
+  Box_Data_List_Related m_HackToFixLinkerError; //The implementation of this class does not seem to be in the library unless I do this. murrayc.
+
 };
 
 #endif //FRAME_GLOM_H
